'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var command = _interopDefault(require('@pagedip/lib-command'));
var inquirer = _interopDefault(require('inquirer'));

var name = "pagedip";

var version = "5.2.0-rc.9";

var main$1 = "Usage: pagedip <command> [--debug] [--silent] [--dev] [--help]\n                         [--version]\n\n  The Pagedip CLI tool.\n\nCommands\n  register                Register this host with a Pagedip License key.\n  install                 Install a Pagedip package by name.\n  run                     Run a Pagedip app.\n  start, server, ee       Start a basic Pagedip Enterprise server.\n\n  This is the main Pagedip CLI tool. With this tool you can install Pagedip\n  packages, run servers, and more.\n\n  The following app names can be used as commands directly: api, auth,\n  couchdb-setup, editor, enterprise-edition, live, setup, theme, theme-server\n\n  Run --help on any command for further help documentation.\n\nOptions\n  --debug                 Show debug messages.\n  -s, --silent            All logs will be hidden.\n  --dev                   Run in development mode.\n  -h, -H, --help          Shows this message.\n  -v, -V, --version       Prints the version and exits.";

var register$1 = "Usage: pagedip register <license> [--help]\n\n  Register this host with a Pagedip License key.\n\n  Install a Pagedip License key onto this host. Once installed, the license is\n  detected by Pagedip server instances and used to verify the legitimacy of the\n  instance.\n\nOptions\n  -h, -H, --help          Shows this message.";

var install$1 = "Usage: pagedip install [<pkgname>...] [--help]\n\n  Install Pagedip packages.\n\n  This will install Pagedip packages and apps from the Pagedip registry. This\n  host should already be registered with a valid license key before installing\n  packages.\n\n  One or more packages can be installed at once. Postfix the package name with\n  @version to install a specific version.\n\nOptions\n  -h, -H, --help          Shows this message.";

var run$1 = "Usage: pagedip run <pkgname> [--help]\n\n  Run Pagedip apps.\n\n  Run a Pagedip app by name. CLI options are passed through to the app\n  directly. This host should already be registered with a valid license key\n  before running apps.\n\n  If the app is not installed on the host, you will be asked to install the app\n  before it runs.\n\nOptions\n  -h, -H, --help          Shows this message.";

var apps = [".babelrc", ".template", "api", "auth", "couchdb-setup", "editor", "enterprise-edition", "live", "setup", "theme", "theme-server"];

var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();



var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};

var install = function () {
  var _ref2 = asyncToGenerator(function* (args, config, logger) {
    if (!args.length || this.help()) {
      logger.info(install$1);
      return;
    }

    yield require("./").install(args, { logger: logger });
  });

  return function install(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();

var run = function () {
  var _ref3 = asyncToGenerator(function* (args, config, logger) {
    if (!args.length) {
      logger.info(run$1);
      return;
    }

    var _require = require("./"),
        load = _require.load;

    var cmd = void 0;

    try {
      cmd = yield load(args[0], { logger: logger });
    } catch (e) {
      if (e.code !== "MODULE_NOT_FOUND") throw e;

      var _ref4 = yield inquirer.prompt([{
        type: "confirm",
        name: "confirm",
        message: "Could not locate a package named '" + args[0] + "'. Do you want to attempt to install it?",
        default: true
      }]),
          confirm = _ref4.confirm;

      if (!confirm) throw e;

      yield install(args[0], config, { logger: logger });
      cmd = yield load(args[0], { logger: logger });
    }

    yield this.run(cmd, args.slice(1), logger);
  });

  return function run(_x7, _x8, _x9) {
    return _ref3.apply(this, arguments);
  };
}();

var register = function () {
  var _ref5 = asyncToGenerator(function* (args, config, logger) {
    if (!args.length || this.help()) {
      logger.info(register$1);
      return;
    }

    yield require("./").register(args[0]);
  });

  return function register(_x10, _x11, _x12) {
    return _ref5.apply(this, arguments);
  };
}();

var cli = command("pagedip", {
  module: module
}, function () {
  var _ref = asyncToGenerator(function* (args, config, logger) {
    var isApp = apps.includes(args[0]);

    if (!isApp && this.version()) {
      logger.info(name + " " + version);
      return;
    }

    switch (args[0]) {
      case "install":
        yield install.call(this, args.slice(1), config, logger);
        break;

      case "run":
        yield run.call(this, args.slice(1), config, logger);
        break;

      case "register":
        yield register.call(this, args.slice(1), config, logger);
        break;

      case "start":
      case "server":
      case "ee":
        {
          var eeArgs = ["enterprise-edition"].concat(args.slice(1));
          yield run.call(this, eeArgs, config, logger);
          break;
        }

      default:
        if (isApp) {
          yield run.call(this, args, config, logger);
        } else {
          logger.info(main$1);
        }
    }
  });

  return function (_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}());

module.exports = cli;
